@@ -14,19 +14,40 @@ use IO::All qw/ io /;
 
 use MooX qw/ late /;
 
-has filename_re => (is => 'ro', default => sub {
+has filename_re => (
+    is      => 'ro',
+    default => sub {
         return qr/\.x?html\z/;
-    });
+    }
+);
 
-has targets => (is => 'ro', isa => 'ArrayRef', required => 1);
+has targets => ( is => 'ro', isa => 'ArrayRef', required => 1 );
 
-has filename_filter => (is => 'ro', default => sub { return sub { return 1; } });
+has filename_filter => (
+    is      => 'ro',
+    default => sub {
+        return sub { return 1; }
+    }
+);
+
+has _tidy => ( is => 'rw' );
+has _error_count => ( is => 'rw', isa => 'Int', default => 0 );
+
+sub report_error
+{
+    my ( $self, $args ) = @_;
+
+    $self->_error_count( 1 + $self->_error_count );
+    diag( $args->{message} );
+
+    return;
+}
 
 sub calc_tidy
 {
     my $self = shift;
 
-    my $tidy = HTML::Tidy->new({ output_xhtml => 1, });
+    my $tidy = HTML::Tidy->new( { output_xhtml => 1, } );
     $tidy->ignore( type => TIDY_WARNING, type => TIDY_INFO );
 
     return $tidy;
@@ -38,40 +59,72 @@ sub run
     plan tests => 1;
     local $SIG{__WARN__} = sub {
         my $w = shift;
-        if ($w !~ /\AUse of uninitialized/)
+        if ( $w !~ /\AUse of uninitialized/ )
         {
             die $w;
         }
         return;
     };
 
-    my $tidy = $self->calc_tidy;
+    $self->_tidy( $self->calc_tidy );
+    $self->traverse;
+    $self->_tidy('NULL');
 
-    my $error_count = 0;
+    # TEST
+    return is( $self->_error_count, 0, "No errors" );
+}
 
-    my $filename_re = $self->filename_re;
-    my $filter = $self->filename_filter;
+sub check_using_tidy
+{
+    my ( $self, $args ) = @_;
 
-    foreach my $target (@{$self->targets})
+    my $fn = $args->{filename};
+
+    $self->_tidy->parse( $fn, ( scalar io->file($fn)->slurp() ) );
+
+    for my $message ( $self->_tidy->messages )
     {
-        for my $fn (File::Find::Object::Rule->file()->name($filename_re)->in($target))
-        {
-            if ($filter->($fn))
+        $self->report_error(
             {
-                $tidy->parse( $fn, (scalar io->file($fn)->slurp()));
+                message => scalar $message->as_string
+            }
+        );
+    }
 
-                for my $message ( $tidy->messages ) {
-                    $error_count++;
-                    diag( $message->as_string);
-                }
+    $self->_tidy->clear_messages();
 
-                $tidy->clear_messages();
+    return;
+}
+
+sub check_file
+{
+    my ( $self, $args ) = @_;
+
+    $self->check_using_tidy($args);
+
+    return;
+}
+
+sub traverse
+{
+    my ($self) = @_;
+    $self->_error_count(0);
+    my $filename_re = $self->filename_re;
+    my $filter      = $self->filename_filter;
+
+    foreach my $target ( @{ $self->targets } )
+    {
+        for my $fn (
+            File::Find::Object::Rule->file()->name($filename_re)->in($target) )
+        {
+            if ( $filter->($fn) )
+            {
+                $self->check_file( { filename => $fn } );
             }
         }
     }
 
-    # TEST
-    is ($error_count, 0, "No errors");
+    return;
 }
 
 1;
@@ -87,7 +140,7 @@ HTML::Tidy .
 
 =head1 VERSION
 
-version v0.0.3
+version 0.2.0
 
 =head1 SYNOPSIS
 
@@ -116,6 +169,10 @@ It was extracted from a bunch of nearly duplicate test scripts in some of
 my (= Shlomi Fish) web sites, as an attempt to avoid duplicate code and
 functionality.
 
+=head1 VERSION
+
+version 0.2.0
+
 =head1 METHODS
 
 =head2 calc_tidy
@@ -135,6 +192,24 @@ or ".xhtml".
 
 The method that runs the program.
 
+=head2 $obj->check_file({filename => $path_string})
+
+Override this method in subclasses to check a file in a different way.
+
+=head2 $obj->check_using_tidy({filename => $path_string})
+
+Actually check a file using tidy. Used by check_file() by default,
+but can also be called there in subclasses.
+
+=head2 $obj->report_error({message => $string});
+
+Reports the error and increment the error count.
+
+=head2 $obj->traverse()
+
+The method that gets called by run() to do the actual traversal of the tree
+without actually checking for no errors. Useful for testing and debugging.
+
 =head2 targets
 
 A parameter that accepts an array reference of targets as strings.
@@ -158,8 +233,7 @@ This is free software, licensed under:
 =head1 BUGS
 
 Please report any bugs or feature requests on the bugtracker website
-http://rt.cpan.org/NoAuth/Bugs.html?Dist=Test-HTML-Tidy-Recursive or by
-email to bug-test-html-tidy-recursive@rt.cpan.org.
+L<https://github.com/shlomif/test-html-tidy-recursive/issues>
 
 When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
